@@ -197,7 +197,8 @@ public class TestDelayEnsembleChange extends BookKeeperClusterTestCase {
             assertTrue(
                     LEDGER_ENSEMBLE_BOOKIE_DISTRIBUTION + " should be > 0 for " + addr,
                     bkc.getTestStatsProvider().getCounter(
-                            CLIENT_SCOPE + "." + LEDGER_ENSEMBLE_BOOKIE_DISTRIBUTION + "-" + addr)
+                            CLIENT_SCOPE + ".bookie_" + addr.toString().replace('-', '_')
+                                    + "." + LEDGER_ENSEMBLE_BOOKIE_DISTRIBUTION)
                             .get() > 0);
         }
         assertTrue(
