@@ -40,6 +40,7 @@ import java.util.concurrent.TimeUnit;
 
 import org.apache.bookkeeper.client.BKException.BKNotEnoughBookiesException;
 import org.apache.bookkeeper.conf.ClientConfiguration;
+import org.apache.bookkeeper.net.BookieId;
 import org.apache.bookkeeper.net.BookieSocketAddress;
 import org.apache.bookkeeper.net.CommonConfigurationKeys;
 import org.apache.bookkeeper.net.DNSToSwitchMapping;
@@ -85,7 +86,8 @@ public class TestRackawareEnsemblePlacementPolicyUsingScript {
                 conf.getTimeoutTimerNumTicks());
 
         repp = new RackawareEnsemblePlacementPolicy();
-        repp.initialize(conf, Optional.<DNSToSwitchMapping>empty(), timer, DISABLE_ALL, NullStatsLogger.INSTANCE);
+        repp.initialize(conf, Optional.<DNSToSwitchMapping>empty(), timer,
+                DISABLE_ALL, NullStatsLogger.INSTANCE, BookieSocketAddress.LEGACY_BOOKIEID_RESOLVER);
     }
 
     @After
@@ -106,16 +108,16 @@ public class TestRackawareEnsemblePlacementPolicyUsingScript {
         BookieSocketAddress addr4 = new BookieSocketAddress("127.0.0.4", 3181); // /4 rack
 
         // Update cluster
-        Set<BookieSocketAddress> addrs = new HashSet<BookieSocketAddress>();
-        addrs.add(addr1);
-        addrs.add(addr2);
-        addrs.add(addr3);
-        addrs.add(addr4);
-        repp.onClusterChanged(addrs, new HashSet<BookieSocketAddress>());
+        Set<BookieId> addrs = new HashSet<BookieId>();
+        addrs.add(addr1.toBookieId());
+        addrs.add(addr2.toBookieId());
+        addrs.add(addr3.toBookieId());
+        addrs.add(addr4.toBookieId());
+        repp.onClusterChanged(addrs, new HashSet<BookieId>());
         // replace node under r2
-        BookieSocketAddress replacedBookie = repp.replaceBookie(1, 1, 1, null, new ArrayList<>(),
-                                                                addr2, new HashSet<>()).getResult();
-        assertEquals(addr3, replacedBookie);
+        BookieId replacedBookie = repp.replaceBookie(1, 1, 1, null, new ArrayList<>(),
+                                                                addr2.toBookieId(), new HashSet<>()).getResult();
+        assertEquals(addr3.toBookieId(), replacedBookie);
     }
 
     @Test
@@ -127,20 +129,21 @@ public class TestRackawareEnsemblePlacementPolicyUsingScript {
         BookieSocketAddress addr4 = new BookieSocketAddress("127.0.0.4", 3181); // /4 rack
 
         // Update cluster
-        Set<BookieSocketAddress> addrs = new HashSet<BookieSocketAddress>();
-        addrs.add(addr1);
-        addrs.add(addr2);
-        addrs.add(addr3);
-        addrs.add(addr4);
-        repp.onClusterChanged(addrs, new HashSet<BookieSocketAddress>());
+        Set<BookieId> addrs = new HashSet<BookieId>();
+        addrs.add(addr1.toBookieId());
+        addrs.add(addr2.toBookieId());
+        addrs.add(addr3.toBookieId());
+        addrs.add(addr4.toBookieId());
+        repp.onClusterChanged(addrs, new HashSet<BookieId>());
         // replace node under r2
-        Set<BookieSocketAddress> excludedAddrs = new HashSet<BookieSocketAddress>();
-        excludedAddrs.add(addr1);
-        BookieSocketAddress replacedBookie = repp.replaceBookie(1, 1, 1, null, new ArrayList<>(),
-                                                                addr2, excludedAddrs).getResult();
+        Set<BookieId> excludedAddrs = new HashSet<BookieId>();
+        excludedAddrs.add(addr1.toBookieId());
+        BookieId replacedBookie = repp.replaceBookie(1, 1, 1, null, new ArrayList<>(),
+                                                                addr2.toBookieId(), excludedAddrs).getResult();
 
-        assertFalse(addr1.equals(replacedBookie));
-        assertTrue(addr3.equals(replacedBookie) || addr4.equals(replacedBookie));
+        assertFalse(addr1.toBookieId().equals(replacedBookie));
+        assertTrue(addr3.toBookieId().equals(replacedBookie)
+                || addr4.toBookieId().equals(replacedBookie));
     }
 
     @Test
@@ -152,19 +155,19 @@ public class TestRackawareEnsemblePlacementPolicyUsingScript {
         BookieSocketAddress addr4 = new BookieSocketAddress("127.0.0.4", 3181); // /4 rack
 
         // Update cluster
-        Set<BookieSocketAddress> addrs = new HashSet<BookieSocketAddress>();
-        addrs.add(addr1);
-        addrs.add(addr2);
-        addrs.add(addr3);
-        addrs.add(addr4);
-        repp.onClusterChanged(addrs, new HashSet<BookieSocketAddress>());
+        Set<BookieId> addrs = new HashSet<BookieId>();
+        addrs.add(addr1.toBookieId());
+        addrs.add(addr2.toBookieId());
+        addrs.add(addr3.toBookieId());
+        addrs.add(addr4.toBookieId());
+        repp.onClusterChanged(addrs, new HashSet<BookieId>());
         // replace node under r2
-        Set<BookieSocketAddress> excludedAddrs = new HashSet<BookieSocketAddress>();
-        excludedAddrs.add(addr1);
-        excludedAddrs.add(addr3);
-        excludedAddrs.add(addr4);
+        Set<BookieId> excludedAddrs = new HashSet<BookieId>();
+        excludedAddrs.add(addr1.toBookieId());
+        excludedAddrs.add(addr3.toBookieId());
+        excludedAddrs.add(addr4.toBookieId());
         try {
-            repp.replaceBookie(1, 1, 1, null, new ArrayList<BookieSocketAddress>(), addr2, excludedAddrs);
+            repp.replaceBookie(1, 1, 1, null, new ArrayList<BookieId>(), addr2.toBookieId(), excludedAddrs);
             fail("Should throw BKNotEnoughBookiesException when there is not enough bookies");
         } catch (BKNotEnoughBookiesException bnebe) {
             // should throw not BKNotEnoughBookiesException
@@ -187,26 +190,26 @@ public class TestRackawareEnsemblePlacementPolicyUsingScript {
         BookieSocketAddress addr2 = new BookieSocketAddress("127.0.0.2", 3181); // /2 rack
 
         // Update cluster, add node that maps to non-default rack
-        Set<BookieSocketAddress> addrs = new HashSet<BookieSocketAddress>();
-        addrs.add(addr1);
+        Set<BookieId> addrs = new HashSet<BookieId>();
+        addrs.add(addr1.toBookieId());
 
-        repp.onClusterChanged(addrs, new HashSet<BookieSocketAddress>());
+        repp.onClusterChanged(addrs, new HashSet<BookieId>());
 
-        addrs = new HashSet<BookieSocketAddress>();
-        addrs.add(addr0);
-        addrs.add(addr1);
-        addrs.add(addr2);
-        repp.onClusterChanged(addrs, new HashSet<BookieSocketAddress>());
+        addrs = new HashSet<BookieId>();
+        addrs.add(addr0.toBookieId());
+        addrs.add(addr1.toBookieId());
+        addrs.add(addr2.toBookieId());
+        repp.onClusterChanged(addrs, new HashSet<BookieId>());
 
         // replace node under r2
-        Set<BookieSocketAddress> excludedAddrs = new HashSet<BookieSocketAddress>();
-        excludedAddrs.add(addr1);
-        BookieSocketAddress replacedBookie = repp.replaceBookie(1, 1, 1, null, new ArrayList<>(),
-                                                                addr2, excludedAddrs).getResult();
+        Set<BookieId> excludedAddrs = new HashSet<BookieId>();
+        excludedAddrs.add(addr1.toBookieId());
+        BookieId replacedBookie = repp.replaceBookie(1, 1, 1, null, new ArrayList<>(),
+                                                                addr2.toBookieId(), excludedAddrs).getResult();
 
-        assertFalse(addr1.equals(replacedBookie));
-        assertFalse(addr2.equals(replacedBookie));
-        assertTrue(addr0.equals(replacedBookie));
+        assertFalse(addr1.toBookieId().equals(replacedBookie));
+        assertFalse(addr2.toBookieId().equals(replacedBookie));
+        assertTrue(addr0.toBookieId().equals(replacedBookie));
     }
 
     /*
@@ -226,26 +229,26 @@ public class TestRackawareEnsemblePlacementPolicyUsingScript {
         BookieSocketAddress addr2 = new BookieSocketAddress("127.0.0.2", 3181); // /2 rack
 
         // Update cluster, add node that maps to default rack first
-        Set<BookieSocketAddress> addrs = new HashSet<BookieSocketAddress>();
-        addrs.add(addr0);
+        Set<BookieId> addrs = new HashSet<BookieId>();
+        addrs.add(addr0.toBookieId());
 
-        repp.onClusterChanged(addrs, new HashSet<BookieSocketAddress>());
+        repp.onClusterChanged(addrs, new HashSet<BookieId>());
 
-        addrs = new HashSet<BookieSocketAddress>();
-        addrs.add(addr0);
-        addrs.add(addr1);
-        addrs.add(addr2);
-        repp.onClusterChanged(addrs, new HashSet<BookieSocketAddress>());
+        addrs = new HashSet<BookieId>();
+        addrs.add(addr0.toBookieId());
+        addrs.add(addr1.toBookieId());
+        addrs.add(addr2.toBookieId());
+        repp.onClusterChanged(addrs, new HashSet<BookieId>());
 
         // replace node under r2
-        Set<BookieSocketAddress> excludedAddrs = new HashSet<BookieSocketAddress>();
-        excludedAddrs.add(addr1);
-        BookieSocketAddress replacedBookie = repp.replaceBookie(1, 1, 1, null, new ArrayList<>(),
-                                                                addr2, excludedAddrs).getResult();
+        Set<BookieId> excludedAddrs = new HashSet<BookieId>();
+        excludedAddrs.add(addr1.toBookieId());
+        BookieId replacedBookie = repp.replaceBookie(1, 1, 1, null, new ArrayList<>(),
+                                                                addr2.toBookieId(), excludedAddrs).getResult();
 
-        assertFalse(addr1.equals(replacedBookie));
-        assertFalse(addr2.equals(replacedBookie));
-        assertTrue(addr0.equals(replacedBookie));
+        assertFalse(addr1.toBookieId().equals(replacedBookie));
+        assertFalse(addr2.toBookieId().equals(replacedBookie));
+        assertTrue(addr0.toBookieId().equals(replacedBookie));
     }
 
     @Test
@@ -256,17 +259,17 @@ public class TestRackawareEnsemblePlacementPolicyUsingScript {
         BookieSocketAddress addr3 = new BookieSocketAddress("127.0.2.1", 3181); // /1 rack
         BookieSocketAddress addr4 = new BookieSocketAddress("127.0.3.1", 3181); // /1 rack
         // Update cluster
-        Set<BookieSocketAddress> addrs = new HashSet<BookieSocketAddress>();
-        addrs.add(addr1);
-        addrs.add(addr2);
-        addrs.add(addr3);
-        addrs.add(addr4);
-        repp.onClusterChanged(addrs, new HashSet<BookieSocketAddress>());
+        Set<BookieId> addrs = new HashSet<BookieId>();
+        addrs.add(addr1.toBookieId());
+        addrs.add(addr2.toBookieId());
+        addrs.add(addr3.toBookieId());
+        addrs.add(addr4.toBookieId());
+        repp.onClusterChanged(addrs, new HashSet<BookieId>());
         try {
-            List<BookieSocketAddress> ensemble = repp.newEnsemble(3, 2, 2, null,
+            List<BookieId> ensemble = repp.newEnsemble(3, 2, 2, null,
                                                                   new HashSet<>()).getResult();
             assertEquals(0, getNumCoveredWriteQuorums(ensemble, 2));
-            List<BookieSocketAddress> ensemble2 = repp.newEnsemble(4, 2, 2, null,
+            List<BookieId> ensemble2 = repp.newEnsemble(4, 2, 2, null,
                                                                    new HashSet<>()).getResult();
             assertEquals(0, getNumCoveredWriteQuorums(ensemble2, 2));
         } catch (BKNotEnoughBookiesException bnebe) {
@@ -282,18 +285,18 @@ public class TestRackawareEnsemblePlacementPolicyUsingScript {
         BookieSocketAddress addr3 = new BookieSocketAddress("127.0.1.2", 3181); // /2 rack
         BookieSocketAddress addr4 = new BookieSocketAddress("127.0.2.2", 3181); // /2 rack
         // Update cluster
-        Set<BookieSocketAddress> addrs = new HashSet<BookieSocketAddress>();
-        addrs.add(addr1);
-        addrs.add(addr2);
-        addrs.add(addr3);
-        addrs.add(addr4);
-        repp.onClusterChanged(addrs, new HashSet<BookieSocketAddress>());
+        Set<BookieId> addrs = new HashSet<BookieId>();
+        addrs.add(addr1.toBookieId());
+        addrs.add(addr2.toBookieId());
+        addrs.add(addr3.toBookieId());
+        addrs.add(addr4.toBookieId());
+        repp.onClusterChanged(addrs, new HashSet<BookieId>());
         try {
-            List<BookieSocketAddress> ensemble = repp.newEnsemble(3, 2, 2, null,
+            List<BookieId> ensemble = repp.newEnsemble(3, 2, 2, null,
                                                                   new HashSet<>()).getResult();
             int numCovered = getNumCoveredWriteQuorums(ensemble, 2);
             assertTrue(numCovered == 2);
-            List<BookieSocketAddress> ensemble2 = repp.newEnsemble(4, 2, 2, null,
+            List<BookieId> ensemble2 = repp.newEnsemble(4, 2, 2, null,
                                                                    new HashSet<>()).getResult();
             numCovered = getNumCoveredWriteQuorums(ensemble2, 2);
             assertTrue(numCovered == 2);
@@ -314,21 +317,21 @@ public class TestRackawareEnsemblePlacementPolicyUsingScript {
         BookieSocketAddress addr7 = new BookieSocketAddress("127.0.1.3", 3181); // /3 rack
         BookieSocketAddress addr8 = new BookieSocketAddress("127.0.1.4", 3181); // /4 rack
         // Update cluster
-        Set<BookieSocketAddress> addrs = new HashSet<BookieSocketAddress>();
-        addrs.add(addr1);
-        addrs.add(addr2);
-        addrs.add(addr3);
-        addrs.add(addr4);
-        addrs.add(addr5);
-        addrs.add(addr6);
-        addrs.add(addr7);
-        addrs.add(addr8);
-        repp.onClusterChanged(addrs, new HashSet<BookieSocketAddress>());
+        Set<BookieId> addrs = new HashSet<BookieId>();
+        addrs.add(addr1.toBookieId());
+        addrs.add(addr2.toBookieId());
+        addrs.add(addr3.toBookieId());
+        addrs.add(addr4.toBookieId());
+        addrs.add(addr5.toBookieId());
+        addrs.add(addr6.toBookieId());
+        addrs.add(addr7.toBookieId());
+        addrs.add(addr8.toBookieId());
+        repp.onClusterChanged(addrs, new HashSet<BookieId>());
         try {
-            List<BookieSocketAddress> ensemble1 = repp.newEnsemble(3, 2, 2, null,
+            List<BookieId> ensemble1 = repp.newEnsemble(3, 2, 2, null,
                                                                    new HashSet<>()).getResult();
             assertEquals(3, getNumCoveredWriteQuorums(ensemble1, 2));
-            List<BookieSocketAddress> ensemble2 = repp.newEnsemble(4, 2, 2, null,
+            List<BookieId> ensemble2 = repp.newEnsemble(4, 2, 2, null,
                                                                    new HashSet<>()).getResult();
             assertEquals(4, getNumCoveredWriteQuorums(ensemble2, 2));
         } catch (BKNotEnoughBookiesException bnebe) {
@@ -348,14 +351,14 @@ public class TestRackawareEnsemblePlacementPolicyUsingScript {
         BookieSocketAddress addr3 = new BookieSocketAddress("127.0.1.2", 3181); // /2 rack
         BookieSocketAddress addr4 = new BookieSocketAddress("127.0.0.4", 3181); // /4 rack
         // Update cluster
-        Set<BookieSocketAddress> addrs = new HashSet<BookieSocketAddress>();
-        addrs.add(addr1);
-        addrs.add(addr2);
-        addrs.add(addr3);
-        addrs.add(addr4);
-        repp.onClusterChanged(addrs, new HashSet<BookieSocketAddress>());
-        addrs.remove(addr1);
-        repp.onClusterChanged(addrs, new HashSet<BookieSocketAddress>());
+        Set<BookieId> addrs = new HashSet<BookieId>();
+        addrs.add(addr1.toBookieId());
+        addrs.add(addr2.toBookieId());
+        addrs.add(addr3.toBookieId());
+        addrs.add(addr4.toBookieId());
+        repp.onClusterChanged(addrs, new HashSet<BookieId>());
+        addrs.remove(addr1.toBookieId());
+        repp.onClusterChanged(addrs, new HashSet<BookieId>());
     }
 
     @Test
@@ -373,7 +376,7 @@ public class TestRackawareEnsemblePlacementPolicyUsingScript {
         repp = new RackawareEnsemblePlacementPolicy();
         try {
             repp.initialize(newConf, Optional.<DNSToSwitchMapping> empty(), timer, DISABLE_ALL,
-                    NullStatsLogger.INSTANCE);
+                    NullStatsLogger.INSTANCE, BookieSocketAddress.LEGACY_BOOKIEID_RESOLVER);
         } catch (RuntimeException re) {
             fail("EnforceMinNumRacksPerWriteQuorum is not set, so repp.initialize should succeed even if"
                     + " networkTopologyScriptFileName is empty");
@@ -384,7 +387,7 @@ public class TestRackawareEnsemblePlacementPolicyUsingScript {
         repp = new RackawareEnsemblePlacementPolicy();
         try {
             repp.initialize(newConf, Optional.<DNSToSwitchMapping> empty(), timer, DISABLE_ALL,
-                    NullStatsLogger.INSTANCE);
+                    NullStatsLogger.INSTANCE, BookieSocketAddress.LEGACY_BOOKIEID_RESOLVER);
             fail("EnforceMinNumRacksPerWriteQuorum is set, so repp.initialize should fail if"
                     + " networkTopologyScriptFileName is empty");
         } catch (RuntimeException re) {
@@ -395,7 +398,7 @@ public class TestRackawareEnsemblePlacementPolicyUsingScript {
                 "src/test/resources/networkmappingscript.sh");
         try {
             repp.initialize(newConf, Optional.<DNSToSwitchMapping> empty(), timer, DISABLE_ALL,
-                    NullStatsLogger.INSTANCE);
+                    NullStatsLogger.INSTANCE, BookieSocketAddress.LEGACY_BOOKIEID_RESOLVER);
         } catch (RuntimeException re) {
             fail("EnforceMinNumRacksPerWriteQuorum is set and networkTopologyScriptFileName is not empty,"
                     + " so it should succeed");
@@ -420,13 +423,14 @@ public class TestRackawareEnsemblePlacementPolicyUsingScript {
                 newConf.getTimeoutTimerTickDurationMs(), TimeUnit.MILLISECONDS, newConf.getTimeoutTimerNumTicks());
 
         repp = new RackawareEnsemblePlacementPolicy();
-        repp.initialize(newConf, Optional.<DNSToSwitchMapping> empty(), timer, DISABLE_ALL, NullStatsLogger.INSTANCE);
+        repp.initialize(newConf, Optional.<DNSToSwitchMapping> empty(), timer,
+                DISABLE_ALL, NullStatsLogger.INSTANCE, BookieSocketAddress.LEGACY_BOOKIEID_RESOLVER);
 
         repp.uninitalize();
         repp = new RackawareEnsemblePlacementPolicy();
         try {
             repp.initialize(newConf, Optional.<DNSToSwitchMapping> empty(), timer, DISABLE_ALL,
-                    NullStatsLogger.INSTANCE);
+                    NullStatsLogger.INSTANCE, BookieSocketAddress.LEGACY_BOOKIEID_RESOLVER);
         } catch (RuntimeException re) {
             fail("EnforceMinNumRacksPerWriteQuorum is not set, so repp.initialize should succeed"
                     + " even if mapping.validateConf fails");
@@ -437,7 +441,7 @@ public class TestRackawareEnsemblePlacementPolicyUsingScript {
         repp = new RackawareEnsemblePlacementPolicy();
         try {
             repp.initialize(newConf, Optional.<DNSToSwitchMapping> empty(), timer, DISABLE_ALL,
-                    NullStatsLogger.INSTANCE);
+                    NullStatsLogger.INSTANCE, BookieSocketAddress.LEGACY_BOOKIEID_RESOLVER);
             fail("EnforceMinNumRacksPerWriteQuorum is set, so repp.initialize should fail"
                     + " if mapping.validateConf fails");
         } catch (RuntimeException re) {
@@ -454,14 +458,14 @@ public class TestRackawareEnsemblePlacementPolicyUsingScript {
         repp = new RackawareEnsemblePlacementPolicy();
         try {
             repp.initialize(newConf, Optional.<DNSToSwitchMapping> empty(), timer, DISABLE_ALL,
-                    NullStatsLogger.INSTANCE);
+                    NullStatsLogger.INSTANCE, BookieSocketAddress.LEGACY_BOOKIEID_RESOLVER);
         } catch (RuntimeException re) {
             fail("EnforceMinNumRacksPerWriteQuorum is set, and mapping.validateConf succeeds."
                     + " So repp.initialize should succeed");
         }
     }
 
-    private int getNumCoveredWriteQuorums(List<BookieSocketAddress> ensemble, int writeQuorumSize)
+    private int getNumCoveredWriteQuorums(List<BookieId> ensemble, int writeQuorumSize)
             throws Exception {
         int ensembleSize = ensemble.size();
         int numCoveredWriteQuorums = 0;
@@ -469,8 +473,9 @@ public class TestRackawareEnsemblePlacementPolicyUsingScript {
             Set<String> racks = new HashSet<String>();
             for (int j = 0; j < writeQuorumSize; j++) {
                 int bookieIdx = (i + j) % ensembleSize;
-                BookieSocketAddress addr = ensemble.get(bookieIdx);
-                String hostAddress = addr.getSocketAddress().getAddress().getHostAddress();
+                BookieId addr = ensemble.get(bookieIdx);
+                String hostAddress = repp.bookieAddressResolver.resolve(addr)
+                                            .getSocketAddress().getAddress().getHostAddress();
                 String rack = "/" + hostAddress.charAt(hostAddress.length() - 1);
                 racks.add(rack);
             }
